{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Cogject demonstration program - load all cogs through a common section of hub memory
and save program space

To run, copy all .cog files to an SD card

Minimalist SD card loader program - disable RTC by setting the RTC constants to -1
Take FullDuplexSerial and remove the spin code and save as Cog_FullDuplexSerial.
Compile to a binary and rename with shorter filename serial.cog and copy to SD card
Take FullDuplexSerial and remove the cog code and save as Spin_FullDuplexSerial.
  Comment out cognew and cogstop and replace cognew with okay := rx_head so returns pointer to the parameter list
Add   Serial: "Spin_FullDuplexSerial.spin"    to the OBJ section
Temporarily add VT100 display driver to OBJ section for debugging
Add code to open the file and close it
Add string object
Open file and read file into CogArray
Load cog and run
Split Keyboard.spin into Cog_keyboard and Spin_keyboard
Compile cog_keyboard, save as .binary, rename to keyboard.cog, copy to sd card
Modify spin_keyboard so returns location of the parameter list




///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1 ' -1 ifnot installed.
  _SD_CD = -1 ' -1 ifnot installed.

  _RTC_DAT = -1 ' -1 ifnot installed.
  _RTC_CLK = -1 ' -1 ifnot installed.
  

OBJ

  fat: "SD3.01_FATEngine.spin"
  rtc: "DS1307_RTCEngine.spin"
  VT100: "VGA_1024_VT100"       ' VGA VT100 Terminal Driver 
  Serial: "Spin_FullDuplexSerial.spin"
  kb : "Spin_Keyboard.spin" 
 
VAR
    byte CogArray[2048] ' reserve space for cogs to be loaded

PUB Main | i 

  VT100.start(16)                                       ' start the vga driver VGA starts on pin 16
  VT100.startcursor                                     ' start the cursor
  VT100.color(%00001000_11111100)                       ' Blue_White  blue =3/4 power 10, white =full                                  
  VT100.cursorset(5)                                    ' cursor type 5
  VT100.clearscreen                                     ' clear the screen
  printstringcr(string("Cogjects demonstration"))
    
  fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)

  PrintStringCR(string("Testing for SD card"))          ' helpful message if card is out, better than just a blank screen
  fat.mountPartition(0)                                 ' mount the sd card
  ifnot(fat.partitionMounted)
    printstringcr(string("Card Not Mounted "))

  ReadCogFile(string("Serial.Cog"))
  cognew(@cogarray+24, serial.start(31,30,0,115200))    ' first 24 bytes discarded, and no @ in front of parameters as this is the mem location

  ReadCogFile(string("Keyboard.Cog"))
  cognew(@cogarray+24, kb.start(26,27))                 ' first 24 bytes discarded, and no @ in front of parameters as this is the mem location              

  printstringcr(string("Hit a key"))
  
  kb.start(26, 27)                                      'start the keyboard                                   
  i := kb.getkey
  printchar(i)
  crlf

  printstringcr(string("Send out serial data at 115200 baud")) 
   
  serial.str(string("Hello World")) ' send something
  printstringcr(string("Finished"))

PRI ReadCogFile(Filename)
  result := \fat.openFile(Filename, "R") ' create file and open file
  result := \fat.readdata(@cogarray,fat.filesize)  ' read all the data into cogarray
  result := \fat.closefile                       ' close the file
  
PRI PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PRI PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    vt100.str(pstring)                                 ' send to vga screen


PRI PrintChar(c) ' sends c to vga 
    vt100.out(c)                                        ' send to vga
    
PRI CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga
        
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}